package trashplace.btTest1;

import android.app.Activity;
import android.bluetooth.BluetoothAdapter;
import android.bluetooth.BluetoothDevice;
import android.content.Intent;
import android.os.Bundle;
import android.os.Handler;
import android.os.Message;
import android.view.View;
import android.view.View.OnClickListener;
import android.widget.Button;
import android.widget.TextView;
import android.widget.Toast;

public class btTest1Main extends Activity implements OnClickListener {

	// Here follows stuff copied from BlueTerm.

	// Message types sent from the BluetoothReadService Handler
	public static final int					MESSAGE_STATE_CHANGE	= 1;
	public static final int					MESSAGE_READ			= 2;
	public static final int					MESSAGE_WRITE			= 3;
	public static final int					MESSAGE_DEVICE_NAME		= 4;
	public static final int					MESSAGE_TOAST			= 5;

	// Key names received from the BluetoothChatService Handler
	public static final String				DEVICE_NAME				= "device_name";
	public static final String				TOAST					= "toast";

	public static BluetoothSerialService	mSerialService			= null;
	// End of stuff copied from BlueTerm

	private static final int				REQUEST_BT_ENABLE		= 1;
	private static final int				REQUEST_CONNECT_DEVICE	= 2;

	String									sRemoteDeviceMac;
	BluetoothDevice							btDevice;

	Button									bConnect;
	Button									bReqCalibrate;
	Button									bReqParams;
	Button									bRunPID;
	TextView								tvSelectedDevice;
	TextView								tvConnectionStatus;
	TextView								tvLog;

	Boolean									fBluetoothAvailable;
	BluetoothAdapter						mBluetoothAdapter;

	public static dataContainer							mData;
	public static mwcParser								mMWCParser;

	/** Called when the activity is first created. */
	@Override
	public void onCreate(Bundle savedInstanceState) {
		super.onCreate(savedInstanceState);
		setContentView(R.layout.main);

		bConnect = (Button) findViewById(R.id.bConnect);
		bReqCalibrate = (Button) findViewById(R.id.bReqCalibrate);
		bReqParams = (Button) findViewById(R.id.bReqParams);
		bRunPID = (Button) findViewById(R.id.bRunPID);
		tvSelectedDevice = (TextView) findViewById(R.id.tvSelectedDevice);
		tvConnectionStatus = (TextView) findViewById(R.id.connectionStatus);
		tvLog = (TextView) findViewById(R.id.tvLog);

		bConnect.setOnClickListener(this);
		bReqCalibrate.setOnClickListener(this);
		bRunPID.setOnClickListener(this);
		bReqParams.setOnClickListener(this);

		mData = new dataContainer(1024);
		mMWCParser = new mwcParser(0, mData);
		mSerialService = new BluetoothSerialService(this, mHandlerBT, mData);

		// Test if Bluetooth is available
		mBluetoothAdapter = BluetoothAdapter.getDefaultAdapter();
		if (mBluetoothAdapter == null) {
			tvSelectedDevice.setText(R.string.BluetoothNotAvailable);
			fBluetoothAvailable = false;
		} else {
			tvSelectedDevice.setText(R.string.BluetoothNotSelected);
			fBluetoothAvailable = true;
		}

		requestEnableBT(true);
	}

	/*
	 * TODO aus der Doku:
	 * Optionally, your application can also listen for the ACTION_STATE_CHANGED
	 * broadcast Intent, which the system will broadcast whenever the Bluetooth state has changed. This broadcast
	 * contains the extra fields EXTRA_STATE and EXTRA_PREVIOUS_STATE, containing the new and old Bluetooth states,
	 * respectively. Possible values for these extra fields are STATE_TURNING_ON, STATE_ON, STATE_TURNING_OFF, and
	 * STATE_OFF. Listening for this broadcast can be useful to detect changes made to the Bluetooth state while your
	 * app is running.
	 * [http://developer.android.com/guide/topics/wireless/bluetooth.html] 
	 */

	@Override
	protected void onRestart() {
		super.onRestart();
		requestEnableBT(true);
	}

	@Override
	protected void onResume() {
		super.onResume();
		requestEnableBT(false);
	}

	// Updates BT-stat and requests the User to enable BT when askUser==true
	private void requestEnableBT(boolean askUser) {
		if (fBluetoothAvailable) {
			// Test if Bluetooth adapter is enabled
			if (!mBluetoothAdapter.isEnabled()) {
				if (askUser == true) {
					// adapter is disabled, ask user to enable it.
					Intent enableBtIntent = new Intent(BluetoothAdapter.ACTION_REQUEST_ENABLE);
					startActivityForResult(enableBtIntent, REQUEST_BT_ENABLE);
				} else {
					// adapter is disabled but request to enable is not set.
					tvConnectionStatus.setText(R.string.conStatDisabled);
				}
			} else { // adapter is already enabled
				tvConnectionStatus.setText(R.string.conStatNotConnected);
			}
		} else { // no adapter available
			tvConnectionStatus.setText(R.string.conStatUnavailable);
		}
	}

	// Resulthandler for BT-Enable-Request.
	@Override
	protected void onActivityResult(int requestCode, int resultCode, Intent data) {
		super.onActivityResult(requestCode, resultCode, data);

		// Returned from "request bt enable" Activity
		switch (requestCode) {
		case REQUEST_BT_ENABLE:
			if (resultCode == RESULT_OK) {
				tvConnectionStatus.setText(R.string.conStatNotConnected);
			} else {
				tvConnectionStatus.setText(R.string.conStatDisabled);
			}
			break;

		case REQUEST_CONNECT_DEVICE:
			requestEnableBT(true); // make sure BT is active

			// check if already connected, then disconnect
			if (mSerialService.getState() == BluetoothSerialService.STATE_CONNECTED) {
				mSerialService.stop();
			} else { // otherwise connect
				sRemoteDeviceMac = data.getExtras().getString(DeviceListActivity.EXTRA_DEVICE_ADDRESS);
				btDevice = mBluetoothAdapter.getRemoteDevice(sRemoteDeviceMac);
				tvSelectedDevice.setText(mBluetoothAdapter.getRemoteDevice(sRemoteDeviceMac).getName()
						+ " (" + sRemoteDeviceMac + ")");
				mSerialService.connect(btDevice); // connect to device.
			}
			break;
		}
	}

	// onClick-handler f. Buttons
	@Override
	public void onClick(View v) {
		switch (v.getId()) {
		case R.id.bConnect:
			//tvSelectedDevice.setText("Select Device clicked.");
			Intent serverIntent = new Intent(this, DeviceListActivity.class);
			startActivityForResult(serverIntent, REQUEST_CONNECT_DEVICE);
			break;

		case R.id.bReqCalibrate:
			// Send Command if device connected.
			if (mSerialService.getState() == BluetoothSerialService.STATE_CONNECTED) {
				mData.clear();
				tvLog.setText("");
				mSerialService.write("D".getBytes());
			}
			break;

		case R.id.bReqParams:
			// Send Command if device connected.
			if (mSerialService.getState() == BluetoothSerialService.STATE_CONNECTED) {
				mData.clear();
				tvLog.setText("");
				mSerialService.write("A".getBytes());
			}
			break;

		case R.id.bRunPID:
			// Invoke PIDSettings Activity
			Intent PIDIntent = new Intent(this, PIDSettings.class);
			Bundle b = new Bundle();
			startActivity(PIDIntent);
		}
	}

	// The Handler that gets information back from the BluetoothService
	private final Handler	mHandlerBT	= new Handler() {

											@Override
											public void handleMessage(Message msg) {
												switch (msg.what) {
												case MESSAGE_STATE_CHANGE:
													tvConnectionStatus.setText(msg.arg1 + ":" + msg.arg2);
													switch (msg.arg1) {
													case BluetoothSerialService.STATE_CONNECTED:
														tvConnectionStatus.setText(R.string.conStatConnected);
														break;
													case BluetoothSerialService.STATE_CONNECTING:
														tvConnectionStatus.setText(R.string.conStatConnecting);
														break;
													case BluetoothSerialService.STATE_LISTEN:
													case BluetoothSerialService.STATE_NONE:
														tvConnectionStatus.setText(R.string.conStatNotConnected);
														break;
													}
													break;
												case MESSAGE_WRITE:
													//tvLog.append("\ndata sent.");
													break;
												case MESSAGE_READ:

													if (mMWCParser.isValid()) {
														//int i;
														//String s = "";
														//byte[] lbuffer = mData.getBuffer();
														//tvLog.append("\ndata read, Count: " + lbuffer.length
														//		+ "; Buffer:\n");
														//for (i = 0; i < lbuffer.length; i++) {
														//	s = String.format("%02x", lbuffer[i]);
														//	tvLog.append(s + " ");
														//}

														mwcParser m = mMWCParser; // create local reference for shorter code

														m.parseData();
														
														PIDSettings.updateGUI();
														
														tvLog.append("Type:  " + m.modelName[m.modelType] + "\n");
														tvLog.append("Accel: " + m.accSmooth[0] + ":" + m.accSmooth[1]
																+ ":" + m.accSmooth[2] + "\n");
														tvLog.append("Gyro:  " + m.gyroData[0] + ":" + m.gyroData[1]
																+ ":" + m.gyroData[2] + "\n");
														tvLog.append("RC:    " + m.rcHysteresis[0] + ":"
																+ m.rcHysteresis[1] + ":"
																+ m.rcHysteresis[2] + ":" + m.rcHysteresis[3]
																+ "\n       "
																+ m.rcHysteresis[4] + ":" + m.rcHysteresis[5] + ":"
																+ m.rcHysteresis[6] + ":" + m.rcHysteresis[7] + "\n");
														tvLog.append("Motors: " + m.motor[0] + ":" + m.motor[1] + ":"
																+ m.motor[2] + ":"
																				+ m.motor[3] + ":" + m.motor[4] + ":"
																+ m.motor[5] + "\n");
														tvLog.append("Cycle Time: " + m.cycleTime + "\n");
													}
													break;
												case MESSAGE_TOAST:
													Toast.makeText(getApplicationContext(),
															msg.getData().getString(TOAST),
															Toast.LENGTH_SHORT).show();
													break;
												}
											}
										};
	// End handler ==============================

}
